@@ -329,12 +329,14 @@ let relocate_cmts_before (t : t) ~src ~sep ~dst =
   update_cmts t `Before ~f ; update_cmts t `Within ~f
 
 let relocate_pattern_matching_cmts (t : t) src tok ~whole_loc ~matched_loc =
-  let kwd_loc =
-    Option.value_exn (Source.loc_of_first_token_at src whole_loc tok)
-  in
-  relocate_cmts_before t ~src:matched_loc ~sep:kwd_loc ~dst:whole_loc
+  if not (whole_loc.Location.loc_ghost || matched_loc.Location.loc_ghost)
+  then
+    let kwd_loc =
+      Option.value_exn (Source.loc_of_first_token_at src whole_loc tok)
+    in
+    relocate_cmts_before t ~src:matched_loc ~sep:kwd_loc ~dst:whole_loc
 
-let relocate_ext_cmts (t : t) src (pre, pld) ~whole_loc =
+let relocate_ext_cmts (t : t) src (_pre, pld) ~whole_loc =
   let open Extended_ast in
   match pld with
   | PStr
@@ -348,37 +350,23 @@ let relocate_ext_cmts (t : t) src (pre, pld) ~whole_loc =
         ; pstr_loc } ]
     when Source.is_quoted_string src pstr_loc ->
       ()
-  | PStr
-      [ { pstr_desc=
-            Pstr_eval
-              ( { pexp_desc= Pexp_sequence (e1, _)
-                ; pexp_loc= _
-                ; pexp_loc_stack= _
-                ; pexp_attributes }
-              , [] )
-        ; pstr_loc= _ } ]
-    when List.is_empty pexp_attributes
-         && Source.extension_using_sugar ~name:pre ~payload:e1.pexp_loc ->
-      ()
   | PStr [{pstr_desc= Pstr_eval _; pstr_loc; _}] ->
       let kwd_loc =
         match Source.loc_of_first_token_at src whole_loc LBRACKETPERCENT with
-        | Some loc -> loc
-        | None -> (
-          match Source.loc_of_first_token_at src whole_loc PERCENT with
-          | Some loc -> loc
-          | None -> impossible "expect token starting extension" )
+        | Some _ as o -> o
+        | None -> Source.loc_of_first_token_at src whole_loc PERCENT
       in
-      relocate_cmts_before t ~src:pstr_loc ~sep:kwd_loc ~dst:whole_loc
+      Option.iter kwd_loc ~f:(fun kwd_loc ->
+          relocate_cmts_before t ~src:pstr_loc ~sep:kwd_loc ~dst:whole_loc )
   | _ -> ()
 
 let relocate_wrongfully_attached_cmts t src exp =
   let open Extended_ast in
   match exp.pexp_desc with
-  | Pexp_match (e0, _) ->
+  | Pexp_match (e0, _, _) ->
       relocate_pattern_matching_cmts t src Parser.MATCH
         ~whole_loc:exp.pexp_loc ~matched_loc:e0.pexp_loc
-  | Pexp_try (e0, _) ->
+  | Pexp_try (e0, _, _) ->
       relocate_pattern_matching_cmts t src Parser.TRY ~whole_loc:exp.pexp_loc
         ~matched_loc:e0.pexp_loc
   | Pexp_extension ext -> relocate_ext_cmts t src ext ~whole_loc:exp.pexp_loc
@@ -581,18 +569,22 @@ end
 
 let fmt_cmt (conf : Conf.t) cmt ~fmt_code =
   let open Fmt in
-  let parse_comments_as_doc = conf.fmt_opts.ocp_indent_compat.v in
-  let decoded = Cmt.decode ~parse_comments_as_doc cmt in
+  let decoded = Cmt.decode cmt in
   (* TODO: Offset should be computed from location. *)
   let offset = 2 + String.length decoded.prefix in
   let pro = str "(*" $ str decoded.prefix
   and epi = str decoded.suffix $ str "*)" in
+  let fmt_doc txt =
+    Doc.fmt ~pro ~epi ~fmt_code conf ~loc:(Cmt.loc cmt) txt ~offset
+  in
   match decoded.kind with
   | Verbatim txt -> Verbatim.fmt ~pro ~epi txt
-  | Doc txt ->
-      Doc.fmt ~pro ~epi ~fmt_code conf ~loc:(Cmt.loc cmt) txt ~offset
+  | Doc txt -> fmt_doc txt
   | Normal txt ->
-      if conf.fmt_opts.wrap_comments.v then Wrapped.fmt ~pro ~epi txt
+      if
+        conf.fmt_opts.ocp_indent_compat.v && conf.fmt_opts.parse_docstrings.v
+      then fmt_doc txt
+      else if conf.fmt_opts.wrap_comments.v then Wrapped.fmt ~pro ~epi txt
       else Unwrapped.fmt ~pro ~epi txt
   | Code code -> Cinaps.fmt ~pro ~epi ~fmt_code conf ~offset code
   | Asterisk_prefixed lines -> Asterisk_prefixed.fmt ~pro ~epi lines
@@ -603,19 +595,19 @@ let fmt_cmts_aux t (conf : Conf.t) cmts ~fmt_code pos =
   vbox 0 ~name:"cmts"
     (list_pn groups (fun ~prev:_ group ~next ->
          ( match group with
-         | [] -> impossible "previous match"
-         | [cmt] ->
-             let break =
-               fmt_if
-                 ( conf.fmt_opts.ocp_indent_compat.v
-                 && Poly.(pos = Cmt.After)
-                 && String.contains (Cmt.txt cmt) '\n' )
-                 (break_unless_newline 1000 0)
-             in
-             break $ fmt_cmt conf cmt ~fmt_code
-         | group ->
-             list group force_break (fun cmt ->
-                 wrap (str "(*") (str "*)") (str (Cmt.txt cmt)) ) )
+           | [] -> impossible "previous match"
+           | [cmt] ->
+               let break =
+                 fmt_if
+                   ( conf.fmt_opts.ocp_indent_compat.v
+                   && Poly.(pos = Cmt.After)
+                   && String.contains (Cmt.txt cmt) '\n' )
+                   (break_unless_newline 1000 0)
+               in
+               break $ fmt_cmt conf cmt ~fmt_code
+           | group ->
+               list group force_break (fun cmt ->
+                   wrap (str "(*") (str "*)") (str (Cmt.txt cmt)) ) )
          $
          match next with
          | Some (next :: _) ->
