@@ -11,14 +11,6 @@
 
 module Error = Conf_t.Error
 
-let ocaml_version_conv =
-  let parse x =
-    match Ocaml_version.of_string x with
-    | Ok x -> `Ok x
-    | Error (`Msg x) -> `Error x
-  in
-  (parse, Ocaml_version.pp)
-
 type typ = Int | Bool | Ocaml_version | Choice of string list
 
 module UI = struct
@@ -31,6 +23,8 @@ end
 
 open Cmdliner
 
+let ocaml_version_conv = Arg.conv (Ocaml_version.of_string, Ocaml_version.pp)
+
 type kind = Formatting | Operational
 (* type from = [ `Default | `Profile of string * updated_from | `Updated of
    updated_from * from option (* when redundant definition *) ] *)
