@@ -25,6 +25,7 @@ type 'a t =
   | Core_type : core_type t
   | Module_type : module_type t
   | Expression : expression t
+  | Pattern : pattern t
   | Repl_file : repl_file t
   | Documentation : Ocamlformat_odoc_parser.Ast.t t
 
@@ -37,6 +38,7 @@ let of_syntax = function
   | Core_type -> Any Core_type
   | Module_type -> Any Module_type
   | Expression -> Any Expression
+  | Pattern -> Any Pattern
   | Repl_file -> Any Repl_file
   | Documentation -> Any Documentation
 
@@ -50,6 +52,7 @@ let map (type a) (x : a t) (m : Ast_mapper.mapper) : a -> a =
   | Core_type -> m.typ m
   | Module_type -> m.module_type m
   | Expression -> m.expr m
+  | Pattern -> m.pat m
   | Repl_file -> List.map ~f:(m.repl_phrase m)
   | Documentation -> Fn.id
 
@@ -151,6 +154,81 @@ module Parse = struct
           (f, Some t, None)
       | _ -> (f, t, Option.map ~f:(m.pat m) v)
     in
+    let map_labeled_tuple_element m f = function
+      | Lte_simple lte -> f m lte
+      | (Lte_constrained_pun _ | Lte_pun _) as x -> x
+    in
+    let pat_tuple_elt m te =
+      match (te.lte_label, te.lte_elt) with
+      (* [ ~x:x ] -> [ ~x ] *)
+      | ( Some lbl
+        , {ppat_desc= Ppat_var {txt= v_txt; _}; ppat_attributes= []; _} )
+        when String.equal lbl.txt v_txt ->
+          Lte_pun lbl
+      (* [~x:(x : t)] -> [ ~(x : t)] *)
+      | ( Some lbl
+        , { ppat_desc=
+              Ppat_constraint
+                ( { ppat_desc= Ppat_var {txt= v_txt; _}
+                  ; ppat_attributes= []
+                  ; _ }
+                , t )
+          ; ppat_attributes= []
+          ; ppat_loc
+          ; _ } )
+        when String.equal lbl.txt v_txt ->
+          Lte_constrained_pun
+            { loc= {lbl.loc with loc_end= ppat_loc.loc_end}
+            ; label= lbl
+            ; type_constraint= t }
+      | lte_label, pat -> Lte_simple {lte_label; lte_elt= m.pat m pat}
+    in
+    let pat_tuple_elt m lte =
+      map_labeled_tuple_element m pat_tuple_elt lte
+    in
+    let exp_tuple_elt m te =
+      match (te.lte_label, te.lte_elt) with
+      (* [ ~x:x ] -> [ ~x ] *)
+      | ( Some lbl
+        , { pexp_desc= Pexp_ident {txt= Lident v_txt; _}
+          ; pexp_attributes= []
+          ; _ } )
+        when String.equal lbl.txt v_txt ->
+          Lte_pun lbl
+      (* [~x:(x : t)] -> [ ~(x : t)] *)
+      | ( Some lbl
+        , { pexp_desc=
+              Pexp_constraint
+                ( { pexp_desc= Pexp_ident {txt= Lident v_txt; _}
+                  ; pexp_attributes= []
+                  ; _ }
+                , t )
+          ; pexp_attributes= []
+          ; pexp_loc
+          ; _ } )
+        when String.equal lbl.txt v_txt ->
+          Lte_constrained_pun
+            { loc= {lbl.loc with loc_end= pexp_loc.loc_end}
+            ; label= lbl
+            ; type_constraint= Pconstraint t }
+      (* [~x:(x : t1 :> t2)] -> [ ~(x : t1 :> t2)] *)
+      | ( Some lbl
+        , { pexp_desc=
+              Pexp_coerce
+                ({pexp_desc= Pexp_ident {txt= Lident v_txt; _}; _}, bty, tty)
+          ; pexp_attributes= []
+          ; pexp_loc
+          ; _ } )
+        when String.equal lbl.txt v_txt ->
+          Lte_constrained_pun
+            { loc= {lbl.loc with loc_end= pexp_loc.loc_end}
+            ; label= lbl
+            ; type_constraint= Pcoerce (bty, tty) }
+      | lte_label, exp -> Lte_simple {lte_label; lte_elt= m.expr m exp}
+    in
+    let exp_tuple_elt m lte =
+      map_labeled_tuple_element m exp_tuple_elt lte
+    in
     let binding_op (m : Ast_mapper.mapper) b =
       let b' =
         let loc_start = b.pbop_op.loc.loc_start in
@@ -181,6 +259,9 @@ module Parse = struct
               , {ptyp_desc= Ptyp_package pt; ptyp_attributes= []; _} )
         ; _ } as p ->
           {p with ppat_desc= Ppat_unpack (name, Some pt)}
+      | {ppat_desc= Ppat_tuple (l, oc); _} as p ->
+          let l = List.map ~f:(pat_tuple_elt m) l in
+          {p with ppat_desc= Ppat_tuple (l, oc)}
       | p -> Ast_mapper.default_mapper.pat m p
     in
     let expr (m : Ast_mapper.mapper) = function
@@ -195,7 +276,9 @@ module Parse = struct
           let exprs = List.(rev (tl_exn (rev l))) in
           {e with pexp_desc= Pexp_list exprs}
       (* Removing beginend *)
-      | {pexp_desc= Pexp_beginend e'; pexp_attributes= []; _}
+      | { pexp_desc= Pexp_beginend (e', {infix_ext= None; infix_attrs= []})
+        ; pexp_attributes= []
+        ; _ }
         when not preserve_beginend ->
           m.expr m e'
       (* Field alias shorthand *)
@@ -217,22 +300,9 @@ module Parse = struct
              && not (Std_longident.is_monadic_binding longident) ->
           let label_loc = {txt= op; loc= loc_op} in
           {e with pexp_desc= Pexp_infix (label_loc, m.expr m l, m.expr m r)}
-      (* [(module M) : (module T)] -> [(module M : T)] *)
-      | { pexp_desc=
-            Pexp_constraint
-              ( { pexp_desc= Pexp_pack (name, None)
-                ; pexp_attributes= []
-                ; pexp_loc
-                ; _ }
-              , {ptyp_desc= Ptyp_package pt; ptyp_attributes= []; ptyp_loc; _}
-              )
-        ; _ } as p
-        when Migrate_ast.Location.compare_start ptyp_loc pexp_loc > 0 ->
-          (* Match locations to differentiate between the two position for
-             the constraint, we want to shorten the second: - [let _ :
-             (module S) = (module M)] - [let _ = ((module M) : (module
-             S))] *)
-          {p with pexp_desc= Pexp_pack (name, Some pt)}
+      | {pexp_desc= Pexp_tuple l; _} as p ->
+          let l = List.map ~f:(exp_tuple_elt m) l in
+          {p with pexp_desc= Pexp_tuple l}
       | e -> Ast_mapper.default_mapper.expr m e
     in
     Ast_mapper.{default_mapper with expr; pat; binding_op}
@@ -253,6 +323,7 @@ module Parse = struct
     | Core_type -> Parse.core_type ~ocaml_version lexbuf
     | Module_type -> Parse.module_type ~ocaml_version lexbuf
     | Expression -> Parse.expression ~ocaml_version lexbuf
+    | Pattern -> Parse.pattern ~ocaml_version lexbuf
     | Repl_file -> Toplevel_lexer.repl_file ~ocaml_version lexbuf
     | Documentation ->
         let pos = (Location.curr lexbuf).loc_start in
@@ -274,6 +345,7 @@ module Printast = struct
     | Core_type -> core_type
     | Module_type -> module_type
     | Expression -> expression
+    | Pattern -> pattern
     | Repl_file -> repl_file
     | Documentation -> Docstring.dump
 end
