@@ -9,17 +9,17 @@
 (*                                                                        *)
 (**************************************************************************)
 
-val dedup_cmts : 'a Extended_ast.t -> 'a -> Cmt.t list -> Cmt.t list
+type 'a t = 'a Extended_ast.t
+
+val dedup_cmts : 'a t -> 'a -> Cmt.t list -> Cmt.t list
 (** Remove comments that duplicate docstrings (or other comments). *)
 
-val equal :
-  'a Extended_ast.t -> ignore_doc_comments:bool -> Conf.t -> 'a -> 'a -> bool
+val equal : 'a t -> ignore_doc_comments:bool -> Conf.t -> 'a -> 'a -> bool
 (** Compare fragments for equality up to normalization. *)
 
-val diff_docstrings :
-  Conf.t -> Cmt.t list -> Cmt.t list -> (unit, Cmt.error list) Result.t
-(** Difference between two lists of doc comments. *)
-
 val diff_cmts :
   Conf.t -> Cmt.t list -> Cmt.t list -> (unit, Cmt.error list) Result.t
 (** Difference between two lists of comments. *)
+
+val normalize_code : Conf.t -> string -> string
+(** Normalize a code block in docstrings. *)
