@@ -88,3 +88,58 @@ include (
     module type of struct include Ast_407
     end
     with module Location := Ast_407.Location )
+
+module type S = sig
+  type t =
+    | AAaaaaaaaaaaaaaaaaaa
+    | Bbbbbbbbbbbbbbbbbbbbbbbbbbb
+    | Ccccccccccccccccccccccc
+    | Dddddddddddddddddd
+end
+
+module S : sig
+  type t =
+    | AAaaaaaaaaaaaaaaaaaa
+    | Bbbbbbbbbbbbbbbbbbbbbbbbbbb
+    | Ccccccccccccccccccccccc
+    | Dddddddddddddddddd
+end = struct
+  type t =
+    | AAaaaaaaaaaaaaaaaaaa
+    | Bbbbbbbbbbbbbbbbbbbbbbbbbbb
+    | Ccccccccccccccccccccccc
+    | Dddddddddddddddddd
+end
+
+module type Module_type_fail = sig
+  include S
+
+  module F : functor (_ : T) -> sig end
+
+  include S
+end
+
+module A : sig
+  type a
+
+  type b
+end =
+  B
+
+module A : sig
+  type a
+
+  type b
+end = struct
+  type a
+
+  type b
+end
+
+module type S = sig
+  module Make (Config : sig
+    val blah : string
+
+    val boo : int
+  end) : S
+end
