@@ -14,6 +14,15 @@ let () =
        in
        fooooooooooo x )
 
+let () =
+  List.iter
+    (fun
+      some_really_really_really_long_name_that_doesn't_fit_on_the_line ->
+    let x =
+      some_really_really_really_long_name_that_doesn't_fit_on_the_line $ y
+    in
+    fooooooooooo x )
+
 let foooooooooo =
   foooooooooooooooooooooo
   |> Option.bind ~f:(function
